@@ -1,7 +1,7 @@
 /**
  * Starts Farabi... :)
  */
-function StartFarabi(editorId) {
+function startFarabi(editorId) {
 
 	// Cache dialog jQuery references for later use
 	var $help_search_dialog = $("#help_search_dialog");
@@ -18,7 +18,13 @@ function StartFarabi(editorId) {
 		if(selection) {
 			_displayHelp(selection, true);
 		} else {
-			_displayHelp('', true);
+			// Search for token under the cursor
+			var token = cm.getTokenAt(cm.getCursor());
+			if(token.string) {
+				_displayHelp($.trim(token.string), true);
+			} else {
+				_displayHelp('', true);
+			}
 		}
 	}
 
@@ -47,6 +53,7 @@ function StartFarabi(editorId) {
 			}
 			if(bShowDialog) {
 				$help_search_dialog.modal('show');
+				$(".topic", $help_search_dialog).select().focus();
 			}
 		});
 	}
@@ -60,6 +67,18 @@ function StartFarabi(editorId) {
 		CodeMirror.autoLoadMode(cm, modeFile);	
 	}
 
+	function showEditorStats(cm) {
+		var cursor = cm.getCursor();
+		var selection = cm.getSelection();
+		$('#editor_stats').html(
+			'Line: <strong>' + (cursor.line + 1) + '</strong>' +
+			', Column: <strong>' + (cursor.ch + 1) + '</strong>' +
+			', Selected: <strong>' + (selection ? selection.length : 0) + '</strong>' +
+			', Length: <strong>' + cm.getValue().length + '</strong>' +
+			', Lines: <strong>' + cm.lineCount() + '</strong>&nbsp;&nbsp;'
+		);
+	}
+
 	$("#mode_selector").change(function() {
 		var $selectedMode = $(":selected", this);
 		humane.log("Mode changed to " + $selectedMode.html());
@@ -84,21 +103,17 @@ function StartFarabi(editorId) {
 				displayHelp(cm);
 			},
 		},
-		onCursorActivity: function() {
+		onCursorActivity: function(cm) {
+
 			// Highlight active line
-			editor.setLineClass(hlLine, null, null);
-			hlLine = editor.setLineClass(editor.getCursor().line, null, "activeline");
+			cm.setLineClass(hlLine, null, null);
+			hlLine = cm.setLineClass(cm.getCursor().line, null, "activeline");
 
 			// Highlight selection matches
-			editor.matchHighlight("CodeMirror-matchhighlight");
-		},
-		onGutterClick: function(cm, n) {
-			var info = cm.lineInfo(n);
-			if (info.markerText) {
-				cm.clearMarker(n);
-			} else {
-				cm.setMarker(n, "<span style=\"color: #900\">*</span> %N%");
-			}
+			cm.matchHighlight("CodeMirror-matchhighlight");
+
+			// Show editor statistics
+			showEditorStats(cm);
 		}
 	});
 	// Editor is by default Perl
@@ -107,6 +122,8 @@ function StartFarabi(editorId) {
 	// focus!
 	editor.focus();
 
+	// Show editor stats
+	showEditorStats(editor);
 
 	// Highlight active line
 	var hlLine = editor.setLineClass(0, "activeline");
@@ -140,7 +157,7 @@ function StartFarabi(editorId) {
 	});
 
 	$(".perl_critic_button").click(function() {
-		$.post('/perl_critic', {"source": editor.getValue()}, function(violations) {
+		$.post('/perl_critic', {"source": editor.getValue(), "severity": $(':selected', '#critic_severity_selector').val()}, function(violations) {
 			if(violations.length > 0) {
 				humane.log("Found Perl::Critic violations!");
 				var html = '';
@@ -151,8 +168,8 @@ function StartFarabi(editorId) {
 					html += '<br/><strong>Line:</strong><br/>' + violation.line_number;
 					html += '<br/><strong>Explanation:</strong><br/>' + violation.explanation;
 					html += '<br/><strong>Policy:</strong><br/><a target="_blank" href="https://metacpan.org/module/' + policy + '">' + policy + '</a>';
-					html += '<br/><strong>Diagnositcs:</strong><br/>' + violation.diagnostics;
-					html += '<br/><br/>';
+					html += '<br/><strong>Diagnositcs:</strong><br/>' + violation.diagnostics.replace(/\n/g, '<br/>');
+					html += '<hr/>';
 				}
 				$('.results', $perl_critic_dialog).html(html);
 				$perl_critic_dialog.modal("show");
@@ -186,8 +203,26 @@ function StartFarabi(editorId) {
 			});
 		}
 	}).change(function() {
-		_displayHelp($(this).val(), false);
+		_displayHelp($(this).val(), true);
 	});;
+
+	setTimeout(function() {
+		$(".farabi-quote-of-the-day").hide("fade");
+	}, 8000);
+
+	$("#line_numbers_checkbox").change(function() {
+		editor.setOption('lineNumbers', $(this).is(':checked'));
+	});
+
+	$("#tab_size").change(function() {
+		var tabSize = $(this).val();
+		if($.isNumeric(tabSize)) {
+			$(this).parent().parent().removeClass("error");
+			editor.setOption('tabSize', tabSize);
+		} else {
+			$(this).parent().parent().addClass("error");
+		}
+	});
 }
 
 function runOnPerlito(source) {
@@ -227,9 +262,15 @@ function runOnPerlito(source) {
 		start = new Date().getTime();
 		eval(js_source);
 		$logResult.val( $logResult.val() + "Running time: " + (new Date().getTime() - start) + "ms\n");
+		$('a[href="#output-tab"]').tab('show');
 	}
 	catch(err) {
 		$logResult.val("Error:\n" + err + "\nCompilation aborted.\n");
-		//$("#bottom_panel").tabs( "select" , 1 );
+		$('a[href="#log-tab"]').tab('show');
 	}
-}
\ No newline at end of file
+}
+
+// Start Farabi when the document loads
+$(function() {
+	startFarabi("editor");
+});
